<?php
/*
 * Plugin Name: WooCommerce PayPal Payment Gateway
 * Plugin URI: TODO!!!!
 * Description: Accept PayPal payments on your site
 * Author: Brian McManus
 * Author URI: http://bmcmsoftware.com
 * Version: 1.0
 
 
 /*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
add_filter( 'woocommerce_payment_gateways', 'add_gateway_class' );
function add_gateway_class( $gateways ) {
	$gateways[] = 'WC_PayPal_Gateway'; // your class name is here
	return $gateways;
}
 
/*
 * The class itself, please note that it is inside plugins_loaded action hook
 */
add_action( 'plugins_loaded', 'init_gateway_class' );
function init_gateway_class() {
 
	class WC_PayPal_Gateway extends WC_Payment_Gateway {
 
 		/**
 		 * Class constructor, more about it in Step 3
 		 */
 		public function __construct() {
 
 		    $this->id = 'paypal'; // payment gateway plugin ID
 		    $this->icon = ''; // URL of the icon that will be displayed on checkout page near your gateway name
 		    $this->has_fields = true; // in case you need a custom credit card form
 		    $this->method_title = 'PayPal Gateway';
 		    $this->method_description = 'Description of PayPal payment gateway'; // will be displayed on the options page
 		    
 		    // gateways can support subscriptions, refunds, saved payment methods,
 		    // but in this tutorial we begin with simple payments
 		    $this->supports = array(
 		        'products'
 		    );
 		    
 		    // Method with all the options fields
 		    $this->init_form_fields();
 		    
 		    // Load the settings.
 		    $this->enabled = $this->get_option( 'enabled' );
 		    $this->init_settings();
 		    $this->title = $this->get_option( 'title' );
 		    $this->description = $this->get_option( 'description' );
 		    $this->payment_action = $this->get_option('payment_action');
 		    $this->sandbox = 'yes' === $this->get_option( 'sandbox' );
 		    
 		    $this->client_id = $this->sandbox ? $this->get_option( 'sandbox_client_id' ) : $this->get_option( 'client_id' );
 		    $this->secret = $this->sandbox ? $this->get_option( 'sandbox_secret' ) : $this->get_option( 'secret' );
 		    
 		    $this->invoice_id_prefix = $this->get_option( 'invoice_id_prefix' );
 		    $this->show_on_cart = 'yes' === $this->get_option( 'show_on_cart' );
 		    
 		    // This action hook saves the settings
 		    add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
 		    
 		    // We need custom JavaScript to obtain a token
 		    add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );
 		    
 		    // You can also register a webhook here
 		    // add_action( 'woocommerce_api_{webhook name}', array( $this, 'webhook' ) );
 
 		}
 
		/**
 		 * Plugin options, we deal with it in Step 3 too
 		 */
 		public function init_form_fields(){
 
 		    $this->form_fields = array(
 		        'enabled' => array(
 		            'title'       => 'Enable/Disable',
 		            'label'       => 'Enable PayPal Gateway',
 		            'type'        => 'checkbox',
 		            'description' => '',
 		            'default'     => 'no'
 		        ),
 		        'title' => array(
 		            'title'       => 'Title',
 		            'type'        => 'text',
 		            'description' => 'This controls the title which the user sees during checkout.',
 		            'default'     => 'PayPal',
 		            'desc_tip'    => true,
 		        ),
 		        'description' => array(
 		            'title'       => 'Description',
 		            'type'        => 'textarea',
 		            'description' => 'This controls the description which the user sees during checkout.',
 		            'default'     => 'Pay with your PayPal account via our super-cool payment gateway.',
 		        ),
 		        'payment_action' => array(
 		            'title'       => 'Payment Action',
 		            'label' => __('Whether to process as a Sale or Authorization.', 'paypal-gateway'),
 		            'description' => __('Sale will capture the funds immediately when the order is placed.  Authorization will authorize the payment but will not capture the funds.'),
 		            'type'        => 'select',
 		            'class'       => 'wc-enhanced-select',
 		            'options' => array(
 		                'Sale' => 'Sale',
 		                'Authorization' => 'Authorization',
 		                'Order' => 'Order'
 		            ),
 		            'default'     => 'Sale',
 		            'desc_tip' => true
 		        ),
 		        'sandbox' => array(
 		            'title'       => 'Sandbox mode',
 		            'label'       => 'Enable Sandbox Mode',
 		            'type'        => 'checkbox',
 		            'description' => 'Place the payment gateway in sandbox mode using test API keys.',
 		            'default'     => 'yes',
 		            'desc_tip'    => true,
 		        ),
 		        'sandbox_client_id' => array(
 		            'title'       => 'Sandbox Client Id',
 		            'type'        => 'text'
 		        ),
 		        'sandbox_secret' => array(
 		            'title'       => 'Sandbox Secret',
 		            'type'        => 'password',
 		        ),
 		        'live_client_id' => array(
 		            'title'       => 'Live Client Id',
 		            'type'        => 'text'
 		        ),
 		        'live_secret' => array(
 		            'title'       => 'Live Secret',
 		            'type'        => 'password'
 		        ),
 		        'invoice-id-prefix' => array(
 		            'title'       => 'Invoice Id Prefix',
 		            'type'        => 'text'
 		        ),
 		        'show_on_cart' => array(
 		            'title'       => 'Show On Cart Page',
 		            'label'       => 'Show paypal on cart page',
 		            'type'        => 'checkbox',
 		            'description' => 'Allow user to checkout faster from the cart.',
 		            'default'     => 'yes',
 		            'desc_tip'    => true,
 		        )
 		    );
 
	 	}
 
// 		/**
// 		 * You will need it if you want your custom credit card form, Step 4 is about it
// 		 */
// 		public function payment_fields() {
 
// 		...
 
// 		}
 
		/*
		 * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
		 */
	 	public function payment_scripts() {
 
		//TODO
 
	 	}
 
// 		/*
//  		 * Fields validation, more in Step 5
// 		 */
// 		public function validate_fields() {
 
// 		...
 
// 		}
 
		/*
		 * We're processing the payments here, everything about it is in Step 5
		 */
		public function process_payment( $order_id ) {
 
		    global $woocommerce;
		    
		    // we need it to get any order details
		    $order = wc_get_order( $order_id );
		    
// 		    //Specify the return/cancel urls
// 		    $urls ['return_url'] = 'http://178.62.46.229/Example_REST_ECS_V4/addressReview.php';
// 		    $urls ['cancel_url'] = 'http://178.62.46.229/Example_REST_ECS_V4/REST_V4_Client_Template.php';
		    
// 		    //Specify the payment method used
// 		    $payer['payment_method'] = "paypal";
		    
// 		    //Specify total transaction details
// 		    $details['subtotal'] = '10.00';
// 		    $details['shipping'] = '2.00';
// 		    $details['tax'] = '2.00';
// 		    $details['shipping_discount'] = '-1.00';
		    
// 		    //Specify total amount details
// 		    $amount['total'] = '13.00';
// 		    $amount['currency'] = 'GBP';
// 		    $amount['details'] = $details;
		    
		    
// 		    //Specify line item details
// 		    $item1['quantity'] = '1';
// 		    $item1['name'] = 'itemName1';
// 		    $item1['price'] = '5';
// 		    $item1['currency'] = 'GBP';
// 		    $item1['description'] = 'item 1 description';
// 		    $item1['tax'] = '1';
		    
// 		    $item2['quantity'] = '1';
// 		    $item2['name'] = 'itemName2';
// 		    $item2['price'] = '5';
// 		    $item2['currency'] = 'GBP';
// 		    $item2['description'] = 'item 2 description';
// 		    $item2['tax'] = '1';
		    
// 		    //Put line items in an array structure
// 		    $items = array($item1, $item2);
		    
// 		    //Put transaction amount details into an array
// 		    $transObj1['amount'] = $amount;
		    
// 		    $item_list['items']= $items;
		    
// 		    //Used for ECM
// 		    //$item_list['shipping_address']= $shippingAddress;
		    
// 		    //Build objects to be placed in json structure
// 		    $transObj1['item_list'] = $item_list;
// 		    $transObj1['description'] = 'The payment transaction description.';
// 		    $transObj1['custom'] = 'merchant custom data.';
		    
		    
// 		    //Create json structure to be sent
// 		    $vars ['intent'] = 'sale';
// 		    $vars['payer'] = $payer;
// 		    $vars['transactions'] = array($transObj1);
// 		    $vars ['redirect_urls'] = $urls;
		    		    
		    /*
		     * Array with parameters for API interaction
		     */
		    $args = array(
		        
		        "arg1"=>"blah"
		        
		    );
		    
		    /*
		     * Your API interaction could be built with wp_remote_post()
		     * payment processor endpoint, parameters
		     */
		    $response = add_query_arg( '{https://api.sandbox.paypal.com/v1/payments/payment}', $args );
		    
		    
		    if( !is_wp_error( $response ) ) {
		        
		        $body = json_decode( $response['body'], true );
		        
		        // it could be different depending on your payment processor
		        if ( $body['response']['responseCode'] == 'APPROVED' ) {
		            
		            // we received the payment
		            $order->payment_complete();
		            $order->reduce_order_stock();
		            
		            // some notes to customer (replace true with false to make it private)
		            $order->add_order_note( 'Hey, your order is paid! Thank you!', true );
		            
		            // Empty cart
		            $woocommerce->cart->empty_cart();
		            
		            // Redirect to the thank you page
		            return array(
		                'result' => 'success',
		                'redirect' => $this->get_return_url( $order )
		            );
		            
		        } else {
		            wc_add_notice(  'Please try again!.', 'error' );
		            return;
		        }
		        
		    } else {
		        wc_add_notice(  'Connection error.', 'error' );
		        return;
		    }
 
	 	}
 
		/*
		 * In case you need a webhook, like PayPal IPN etc
		 */
		public function webhook() {
 
		    $order = wc_get_order( $_GET['id'] );
		    $order->payment_complete();
		    $order->reduce_order_stock();
		    
		    update_option('webhook_debug', $_GET);
 
	 	}
 	}
}